@@ -23,9 +23,12 @@
 cim <-
 function(...) UseMethod("cim")
 
+# -------------------------------------
+# CIM default
+# -------------------------------------
 
-cim.default <-
-function(mat, 
+cim.default <-function(
+		mat, 
          breaks, 
          col = jet.colors, 
          distfun = dist, 
@@ -54,7 +57,7 @@ function(mat,
     if (length(dim(mat)) != 2) 
         stop("'mat' must be a numeric matrix.")
 
-    mat = as.matrix(mat)
+    mat = simMat = as.matrix(mat)
 
     if (!is.numeric(mat)) 
         stop("'mat' must be a numeric matrix.")
@@ -266,7 +269,8 @@ function(mat,
         repeat {
             grDevices::dev.set(DEV.x)
 
-            loc = .Internal(locator(1, type = "n"))
+##            loc = .Internal(locator(1, type = "n"))
+            loc = locator(1, type = "n")
             x1 = round(loc[[1]] - 0.5, 0) + 0.5
             y1 = round(loc[[2]] - 0.5, 0) + 0.5
             flag = loc[[3]]
@@ -292,7 +296,8 @@ function(mat,
 
                 points(x1, y1, type = "p", pch = 3, cex = 2)
 
-                loc = .Internal(locator(1, type = "n"))
+##                loc = .Internal(locator(1, type = "n"))
+				loc = locator(1, type = "n")
                 x2 = round(loc[[1]] - 0.5, 0) + 0.5
                 y2 = round(loc[[2]] - 0.5, 0) + 0.5
 
@@ -378,7 +383,7 @@ function(mat,
     }
 
     par(op)
-    res = list(labCol = labCol, labRow = labRow)
+    res = list(simMat = simMat, labCol = labCol, labRow = labRow)
 	
     if (any(dendrogram == "both") || any(dendrogram == "row")) {
         res$rowInd = rowInd
@@ -392,3 +397,246 @@ function(mat,
 	
     return(invisible(res))
 }
+
+
+# -------------------------------------
+# cim for rcc object 
+# -------------------------------------
+
+
+cim.rcc <-
+function(object, 
+         comp = 1, 
+         X.names = NULL, 
+         Y.names = NULL, 
+         ...) 
+{
+
+    p = ncol(object$X)
+    q = ncol(object$Y)
+    ncomp = object$ncomp	
+	
+    if (length(comp) == 1) {
+	    if (is.null(comp) || !is.numeric(comp) || comp <= 0 || comp > ncomp)
+            stop("invalid value for 'comp'.")
+    }
+	
+    if (length(comp) > 1) {
+        if(length(comp) > ncomp) 
+            stop("the length of 'comp' must be smaller or equal than ", ncomp, ".")
+        if (!is.numeric(comp) || any(comp < 1))
+            stop("invalid vector for 'comp'.")
+        if (any(comp > ncomp)) 
+            stop("the elements of 'comp' must be smaller or equal than ", ncomp, ".")
+    }
+	
+    if (length(X.names) != p & !is.null(X.names))
+        stop("'X.names' must be a character vector of length ", p, ".")
+		
+    if (length(Y.names) != q & !is.null(Y.names))
+        stop("'Y.names' must be a character vector of length ", q, ".")
+
+    comp = round(comp)
+
+    if (is.null(X.names)) X.names = object$names$X
+    if (is.null(Y.names)) Y.names = object$names$Y
+
+    bisect = object$variates$X[, comp] + object$variates$Y[, comp]
+    cord.X = cor(object$X, bisect, use = "pairwise")
+    cord.Y = cor(object$Y, bisect, use = "pairwise")
+    simMat = as.matrix(cord.X %*% t(cord.Y))
+
+    if (ncol(simMat) < nrow(simMat)) {
+        simMat = t(simMat)
+        aux.names = X.names
+        X.names = Y.names
+        Y.names = aux.names
+    }  
+	
+    result = cim(simMat, labRow = X.names, labCol = Y.names, ...)
+    return(invisible(result))
+}
+
+# --------------------------------
+# CIM for sPLS object 
+# ---------------------------------
+
+cim.spls <-
+function(object, 
+         comp = 1, 
+         X.names = NULL, 
+         Y.names = NULL, 
+         keep.var = TRUE, 
+         ...) 
+{
+
+    # validation des arguments #
+	#--------------------------#
+    dim = object$ncomp	
+    	
+    if (length(comp) == 1) {
+	    if (is.null(comp) || !is.numeric(comp) || comp <= 0)
+            stop("invalid value for 'comp'.")
+        if (comp > dim) 
+            stop("'comp' must be smaller or equal than ", dim, ".")
+    }
+    	
+    if (length(comp) > 1) {
+        if(length(comp) > dim) 
+            stop("the length of 'comp' must be smaller or equal than ", dim, ".")
+        if (!is.numeric(comp) || any(comp < 1))
+            stop("invalid vector for 'comp'.")
+        if (any(comp > dim)) 
+            stop("the elements of 'comp' must be smaller or equal than ", dim, ".")
+    }
+    	
+    p = ncol(object$X)
+    q = ncol(object$Y)
+		
+    if (length(X.names) != p & !is.null(X.names))
+        stop("'X.names' must be a character vector of length ", p, ".")
+		
+    if (length(Y.names) != q & !is.null(Y.names))
+        stop("'Y.names' must be a character vector of length ", q, ".")
+		
+    comp = round(comp)
+     
+    # Calcul de la matrice des associations entre les variables X et Y #
+    #------------------------------------------------------------------#
+    if (isTRUE(keep.var)) {
+        keep.X = apply(abs(object$loadings$X), 1, sum) > 0
+        keep.Y = apply(abs(object$loadings$Y), 1, sum) > 0
+
+        if (object$mode == "canonical") {
+            cord.X = cor(object$X[, keep.X], object$variates$X[, comp], 
+                     use = "pairwise")
+            cord.Y = cor(object$Y[, keep.Y], object$variates$Y[, comp], 
+                     use = "pairwise")
+        }
+        else {
+            cord.X = cor(object$X[, keep.X], object$variates$X[, comp], 
+                     use = "pairwise")
+            cord.Y = cor(object$Y[, keep.Y], object$variates$X[, comp], 
+                     use = "pairwise")
+        }		
+		
+        if (is.null(X.names)) X.names = object$names$X[keep.X]
+	    if (is.null(Y.names)) Y.names = object$names$Y[keep.Y]
+    }
+    else {
+        if (object$mode == "canonical") {
+            cord.X = cor(object$X, object$variates$X[, comp], use = "pairwise")
+            cord.Y = cor(object$Y, object$variates$Y[, comp], use = "pairwise")
+        }
+        else {
+            cord.X = cor(object$X, object$variates$X[, comp], use = "pairwise")
+            cord.Y = cor(object$Y, object$variates$X[, comp], use = "pairwise")
+        }
+     		
+        if (is.null(X.names)) X.names = object$names$X
+        if (is.null(Y.names)) Y.names = object$names$Y
+    }
+     	
+    simMat = cord.X %*% t(cord.Y)
+    if (ncol(simMat) < nrow(simMat)) {
+        simMat = t(simMat)
+        aux.names = X.names
+        X.names = Y.names
+        Y.names = aux.names
+    }  
+     	
+    result = cim(simMat, labRow = X.names, labCol = Y.names, ...)
+    return(invisible(result))
+}
+
+# ---------------------------------
+# CIM for PLS object 
+# ---------------------------------
+cim.pls <-
+function(object, 
+         comp = 1, 
+         X.names = NULL, 
+         Y.names = NULL, 
+         ##keep.var = TRUE, 
+         ...) 
+{
+
+    # validation des arguments #
+	#--------------------------#
+    dim = object$ncomp	
+    	
+    if (length(comp) == 1) {
+	    if (is.null(comp) || !is.numeric(comp) || comp <= 0)
+            stop("invalid value for 'comp'.")
+        if (comp > dim) 
+            stop("'comp' must be smaller or equal than ", dim, ".")
+    }
+    	
+    if (length(comp) > 1) {
+        if(length(comp) > dim) 
+            stop("the length of 'comp' must be smaller or equal than ", dim, ".")
+        if (!is.numeric(comp) || any(comp < 1))
+            stop("invalid vector for 'comp'.")
+        if (any(comp > dim)) 
+            stop("the elements of 'comp' must be smaller or equal than ", dim, ".")
+    }
+    	
+    p = ncol(object$X)
+    q = ncol(object$Y)
+		
+    if (length(X.names) != p & !is.null(X.names))
+        stop("'X.names' must be a character vector of length ", p, ".")
+		
+    if (length(Y.names) != q & !is.null(Y.names))
+        stop("'Y.names' must be a character vector of length ", q, ".")
+		
+    comp = round(comp)
+     
+    # Calcul de la matrice des associations entre les variables X et Y #
+    #------------------------------------------------------------------#
+#    if (isTRUE(keep.var)) {
+#        keep.X = apply(abs(object$loadings$X), 1, sum) > 0
+#        keep.Y = apply(abs(object$loadings$Y), 1, sum) > 0
+
+#        if (object$mode == "canonical") {
+#            cord.X = cor(object$X[, keep.X], object$variates$X[, comp], 
+#                     use = "pairwise")
+#            cord.Y = cor(object$Y[, keep.Y], object$variates$Y[, comp], 
+#                     use = "pairwise")
+#        }
+#        else {
+#            cord.X = cor(object$X[, keep.X], object$variates$X[, comp], 
+#                     use = "pairwise")
+#            cord.Y = cor(object$Y[, keep.Y], object$variates$X[, comp], 
+#                     use = "pairwise")
+#        }		
+		
+#        if (is.null(X.names)) X.names = object$names$X[keep.X]
+#	    if (is.null(Y.names)) Y.names = object$names$Y[keep.Y]
+#    }
+#    else {
+        if (object$mode == 'canonical'){
+            cord.X = cor(object$X, object$variates$X[, comp], use = 'pairwise')
+            cord.Y = cor(object$Y, object$variates$Y[, comp], use = 'pairwise')
+        }
+        else{
+            cord.X = cor(object$X, object$variates$X[, comp], use = 'pairwise')
+            cord.Y = cor(object$Y, object$variates$X[, comp], use = 'pairwise')
+        }
+     		
+        if (is.null(X.names)) X.names = object$names$X
+        if (is.null(Y.names)) Y.names = object$names$Y
+#    }
+     	
+    simMat = cord.X %*% t(cord.Y)
+    if (ncol(simMat) < nrow(simMat)) {
+        simMat = t(simMat)
+        aux.names = X.names
+        X.names = Y.names
+        Y.names = aux.names
+    }  
+     	
+    result = cim(simMat, labRow = X.names, labCol = Y.names, ...)
+    return(invisible(result))
+}
+
